#include <stdio.h>
#include <math.h>
#include <algorithm>

void interpolation(double* rhs_x, double* rhs_y, double* rhs_z, int ncp, double* knot, double* d_x, double* d_y, double* d_z);
void parameters(double* rhs_x, double* rhs_y, double* rhs_z, int n, double* knot);
double distance_p(double x1, double y1, double z1, double x2, double y2, double z2);
void set_up_system(double* knot, int n, double* alpha, double* beta,double* gamma);
void l_u_system(double* alpha, double* beta, double* gamma, int n, double* up, double* low);
void solve_system(double* up, double* low, double* gamma, int n, double* rhs, double* v);
double cox_deboor_recursion(int i, int k, double u, double* knot);

void main()
{
	int i;
	int num_p = 4;
	int ncp = num_p + 2;
	double *p_x = new double[num_p];
	double *p_y = new double[num_p];
	double *p_z = new double[num_p];
	double t_x[2];
	double t_y[2];
	double t_z[2];
	double *d_x = new double[ncp];
	double *d_y = new double[ncp];
	double *d_z = new double[ncp];
	double *knot = new double[ncp+4];
	double *rhs_x = new double[ncp];
	double *rhs_y = new double[ncp];
	double *rhs_z = new double[ncp];
	double r_x = 0;
	double r_y = 0;
	double r_z = 0;

	double u = 1.5;

	p_x[0] = 0;	p_y[0] = 0;	p_z[0] = 0;
	p_x[1] = 1;	p_y[1] = 1;	p_z[1] = 0;
	p_x[2] = 0;	p_y[2] = 2;	p_z[2] = 0;
	p_x[3] = 4;	p_y[3] = 6;	p_z[3] = 0;
	t_x[0] = 1;	t_y[0] = 0;	t_z[0] = 0;
	t_x[1] = 1;	t_y[1] = 1;	t_z[1] = 0;

	printf("[Given]\n\n");
	for(i = 0; i < num_p; i++)
	{
		printf("p[%d] = (%lf, %lf, %lf)\n",i, p_x[i], p_y[i], p_z[i]);
	}
	printf("t[0] = (%lf, %lf, %lf)\n",t_x[0], t_y[0], t_z[0]);
	printf("t[1] = (%lf, %lf, %lf)\n",t_x[1], t_y[1], t_z[1]);
	printf("\n");
	printf("u* = %lf\n\n\n", u);
	
	rhs_x[0] = t_x[0];
	rhs_x[ncp-1] = t_x[1];
	rhs_y[0] = t_y[0];
	rhs_y[ncp-1] = t_y[1];
	rhs_z[0] = t_z[0];
	rhs_z[ncp-1] = t_z[1];

	for(i = 1; i < ncp - 1; i++)
	{
		rhs_x[i] = p_x[i-1];
		rhs_y[i] = p_y[i-1];
		rhs_z[i] = p_z[i-1];
	}
	
	interpolation(rhs_x, rhs_y, rhs_z, ncp, knot, d_x, d_y, d_z);


	for(i = 0 ; i < ncp ; i++) // i:basis function index
	{					
		double N = cox_deboor_recursion(i, 4, u, knot);
		r_x += d_x[i] * N;
		r_y += d_y[i] * N;
		r_z += d_z[i] * N;
	}

	printf("[Solution]\n\n");

//	for(i = 0; i < ncp + 4; i++)
//	{
//		printf("knot[%d] = %lf\n",i, knot[i]);
//	}
//	printf("\n");

	for(i = 0; i < ncp; i++)
	{
		printf("d[%d] = (%lf, %lf, %lf)\n",i, d_x[i], d_y[i], d_z[i]);
	}
	printf("\n");
  
	printf("r(u*) = r(%lf) = (%lf, %lf, %lf)\n\n",u, r_x, r_y, r_z);

	delete[] p_x;
	delete[] p_y;
	delete[] p_z;
	delete[] d_x;
	delete[] d_y;
	delete[] d_z;
	delete[] rhs_x;
	delete[] rhs_y;
	delete[] rhs_z;
	delete[] knot;
}

void interpolation(double* rhs_x, double* rhs_y, double* rhs_z, int ncp, double* knot, double* d_x, double* d_y, double* d_z)
{		
  double* alpha = new double[ncp];
  double* beta = new double[ncp];
  double* gamma = new double[ncp];
  double* up = new double[ncp];
  double* low = new double[ncp];	

  parameters(rhs_x, rhs_y, rhs_z, ncp-1, knot);
  set_up_system(knot, ncp-1, alpha, beta, gamma);
  l_u_system(alpha, beta, gamma, ncp-1, up, low);
  solve_system(up, low, gamma, ncp-1, rhs_x, d_x);
  solve_system(up, low, gamma, ncp-1, rhs_y, d_y);
  solve_system(up, low, gamma, ncp-1, rhs_z, d_z);

  delete[] alpha;
  delete[] beta;
  delete[] gamma;
  delete[] up;
  delete[] low;
}
void parameters(double* rhs_x, double* rhs_y, double* rhs_z, int n, double* knot)
{
	int i;
	int j=3;
	double delta1=0.;
	double delta2=0.;

	for(i=0; i<4 ; i++) knot[i]=0.0;

	knot[4] = 1;
	delta1 = 1;

	//choord length parameterization
	for(i=5; i<=n+1; i++)
	{		
		delta2=delta1*sqrt(distance_p(rhs_x[j],rhs_y[j],rhs_z[j],rhs_x[j-1],rhs_y[j-1],rhs_z[j-1])/distance_p(rhs_x[j-1],rhs_y[j-1],rhs_z[j-1],rhs_x[j-2],rhs_y[j-2],rhs_z[j-2]));
		knot[i]=knot[i-1]+delta2;
		j++;
		delta1=delta2;
	}
	for(i=n+2; i<=n+4; i++) knot[i]=knot[n+1];
}

double distance_p(double x1, double y1, double z1, double x2, double y2, double z2)
{
	return sqrt((x2-x1)*(x2-x1)+(y2-y1)*(y2-y1)+(z2-z1)*(z2-z1));
}

void set_up_system(double* knot, int n, double* alpha, double* beta,double* gamma)
{
  int i;
  double delta_i,delta_ip1, delta_ip2, delta_ip3, sum;	

  /* Now for the main loop: */
  for(i=2; i<=n-2; i++)
  {
  /* compute delta_i,... */
	delta_i = (knot[i+1]-knot[i]);
	delta_ip1 = (knot[i+2]-knot[i+1]);
	delta_ip2 = (knot[i+3]-knot[i+2]);
	delta_ip3 = (knot[i+4]-knot[i+3]);
	
	sum = delta_ip1+delta_ip2;

	alpha[i]=(delta_ip2*delta_ip2/(delta_i +sum) )/sum;
	beta[i] =( delta_ip2*(delta_i+delta_ip1)/(delta_i+sum)+ delta_ip1*(delta_ip2+delta_ip3)/(sum+ delta_ip3))/sum;
	gamma[i]=(delta_ip1*delta_ip1/(sum + delta_ip3))/sum;
  }

  /* special care at the end: */
  beta[0]=1.0; gamma[0]=0.0;
  alpha[1]=-3.0; beta[1]=3.0; gamma[1]=0.0;	
  alpha[n-1]=0.0; beta[n-1]=-3.0; gamma[n-1]=3.0;
  alpha[n]=0.0; beta[n]=1.0;
}

void l_u_system(double* alpha, double* beta, double* gamma, int n, double* up, double* low)
{
  int i;

  up[0]=beta[0];
  for(i=1; i<=n; i++)
  {
	low[i]=alpha[i]/up[i-1];
	up[i] =beta[i]-low[i]*gamma[i-1];
  }
}

void solve_system(double* up, double* low, double* gamma, int n, double* rhs, double* v)
{
  int i;
  double* w = new double[n+1];	

  /* special care at the end: */
  std::swap(rhs[0], rhs[1]);  //std c++ ǥض̺귯 ϴ ̴.
  std::swap(rhs[n-1], rhs[n]);

  /* forward substitution: */
  w[0]=rhs[0];
  for(i=1; i<=n; i++) w[i]=rhs[i]-low[i]*w[i-1];
	
  /* backward substitution: */
  v[n]=w[n]/up[n];	
  for(i=n-1; i>=0; i--) v[i]=(w[i]-gamma[i]*v[i+1])/up[i];		
  delete[] w;

  std::swap(rhs[0], rhs[1]);  //std c++ ǥض̺귯 ϴ ̴.
  std::swap(rhs[n-1], rhs[n]);
}

double cox_deboor_recursion(int i, int k, double u, double* knot)
{
	double N_1, N_2, r_1, r_2, coordinate;

	if(k==1)
	{
		if( knot[i]<= u && u < knot[i+1] ) coordinate = 1;			
		else coordinate =0 ;
	}	
	else
	{
		N_1 = cox_deboor_recursion( i, k-1, u, knot);
		N_2 = cox_deboor_recursion( i+1, k-1, u, knot);

		if ( (knot[i+k-1]-knot[i]) == 0 ) r_1 = 0;
		else r_1 = (u-knot[i])/(knot[i+k-1]-knot[i]);
		if ((knot[i+k]-knot[i+1]) == 0) r_2 = 0 ;
		else r_2 = (knot[i+k]-u)/(knot[i+k]-knot[i+1]);

		coordinate = r_1*N_1 + r_2*N_2;
	}
	return coordinate;  
}
